//
// Author:    Jae Chung
// Date:      10/10/99
// File Name: mm-app-mpeg.h
//

#include "timer-handler.h"
#include "packet.h"
#include "app.h"
#include "udp-mm.h"

// This is used for receiver's received packet accounting
struct pkt_accounting {
        int last_seq;       // sequence number of last received MM pkt
        double last_time;   // local time of last received MM pkt
        int last_scale;     // rate (0-4) of last acked
        int lost_pkts;      // number of lost pkts since last ack
        int recv_pkts;      // number of MM pkts received since last ack
        int tot_recv_pkts;  // number of total MM pkts received
        double rtt;         // round trip time
};


class MmAppMpeg;


// Sender uses this timer to 
// schedule next app data packet transmission time
class MmAppMpegSendTimer : public TimerHandler {
 public:
	MmAppMpegSendTimer(MmAppMpeg* t) : TimerHandler(), t_(t) {}
	inline virtual void expire(Event*);
 protected:
	MmAppMpeg* t_;
};


// Reciver uses this timer to schedule
// next ack packet transmission time
class MmAppMpegAckTimer : public TimerHandler {
 public:
	MmAppMpegAckTimer(MmAppMpeg* t) : TimerHandler(), t_(t) {}
	inline virtual void expire(Event*);
 protected:
	MmAppMpeg* t_;
};


// Mulitmedia Application Class Definition
class MmAppMpeg : public Application {
 public:
	MmAppMpeg();
	void send_mm_pkt();  // called by SendTimer:expire (Sender)
	void send_ack_pkt(); // called by AckTimer:expire (Receiver)
 protected:
	int command(int argc, const char*const* argv);
	void start();       // Start sending data packets (Sender)
	void stop();        // Stop sending data packets (Sender)
 private:
	void init();
	int  get_pkt_size(char* ftype);                         // (Sender)
	void update_recv_pkt_type(char pkt_type);               // (Sender)
	virtual void recv_msg(int nbytes, const char *msg = 0); // (Sender/Receiver)
	void set_scale(const hdr_mm *mh_buf);                   // (Sender)
	void adjust_scale(void);                                // (Receiver)
	void account_recv_pkt(const hdr_mm *mh_buf);            // (Receiver)
	void init_recv_pkt_accounting();                        // (Receiver)

	double interval_;      // Application data packet transmission interval
	double max_interval_;  // Maximum possible transmission interval
	int frame_per_sec_;    // Mpeg Frame Rate
	int pktsize_;          // Application data packet size
	int running_;          // If 1 application is running
	int seq_;              // Application data packet sequence number (sent only)
	int fnum_;             // Frame number (account for not sent frames also)
	int flow_control_;     // Enable or Disable Flow Contorl
	int scale_;            // Media scale parameter

	int sup_mm_;           // Flag indicate that underlying agent support mm

	FILE* fdr_;            // file descriptor for input file
	FILE* fdw_;            // file descriptor for mm trace output file
	int file_closed_;      // input file closed flag

	char recv_pkt_type[9]; // 9 previously read mpeg packet type
	int  p_pkt_sent_;      // P packet drop flag for IBBPBBI at scale 1

	pkt_accounting p_accnt;         // Receiver side packet accounting
	MmAppMpegSendTimer snd_timer_;  // SendTimer
	MmAppMpegAckTimer  ack_timer_;  // AckTimer
};



