//
// Author:    Jae Chung
// Date:      10/05/99
// File Name: mm-app.h
//

#include "timer-handler.h"
#include "packet.h"
#include "app.h"
#include "udp-mm.h"

// This is used for receiver's received packet accounting
struct pkt_accounting {
        int last_seq;       // sequence number of last received MM pkt
        double last_time;   // local time of last received MM pkt
        int last_scale;     // rate (0-4) of last acked
        int lost_pkts;      // number of lost pkts since last ack
        int recv_pkts;      // number of MM pkts received since last ack
        int tot_recv_pkts;  // number of total MM pkts received
        double rtt;         // round trip time
};


class MmApp;


// Sender uses this timer to 
// schedule next app data packet transmission time
class MmAppSendTimer : public TimerHandler {
 public:
	MmAppSendTimer(MmApp* t) : TimerHandler(), t_(t) {}
	inline virtual void expire(Event*);
 protected:
	MmApp* t_;
};


// Reciver uses this timer to schedule
// next ack packet transmission time
class MmAppAckTimer : public TimerHandler {
 public:
	MmAppAckTimer(MmApp* t) : TimerHandler(), t_(t) {}
	inline virtual void expire(Event*);
 protected:
	MmApp* t_;
};


// Mulitmedia Application Class Definition
class MmApp : public Application {
 public:
	MmApp();
	void send_mm_pkt();  // called by SendTimer:expire (Sender)
	void send_ack_pkt(); // called by AckTimer:expire (Receiver)
 protected:
	int command(int argc, const char*const* argv);
	void start();       // Start sending data packets (Sender)
	void stop();        // Stop sending data packets (Sender)
 private:
	void init();
	inline double next_snd_time();                          // (Sender)
	virtual void recv_msg(int nbytes, const char *msg = 0); // (Sender/Receiver)
	void set_scale(const hdr_mm *mh_buf);                   // (Sender)
	void adjust_scale(void);                                // (Receiver)
	void account_recv_pkt(const hdr_mm *mh_buf);            // (Receiver)
	void init_recv_pkt_accounting();                        // (Receiver)

	double rate[5];        // Transmission rates associated to scale values
	double interval_;      // Application data packet transmission interval
	double max_interval_;  // Maximum possible transmission interval
	int flow_control_;     // Enable or disable flow control (if disabled use rate[4])
	int pktsize_;          // Application data packet size
	int random_;           // If 1 add randomness to the interval
	int running_;          // If 1 application is running
	int seq_;              // Application data packet sequence number
	int scale_;            // Media scale parameter

	FILE* fdw_;

	pkt_accounting p_accnt;
	MmAppSendTimer snd_timer_;  // SendTimer
	MmAppAckTimer  ack_timer_;  // AckTimer
};



